/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Dict
 *
 * The Initial Developer of the Original Code is David Costanzo.
 * Portions created by the Initial Developer are Copyright (C) 2004 - 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
"use strict";

Components.utils.import("chrome://dict/content/dictclient.js");

var EXPORTED_SYMBOLS = ['DictExtension'];

if ("undefined" == typeof(DictExtension)) {
    var DictExtension = {};
};

// The preferences that we handle
DictExtension.PrefName = {
    DEFAULTSERVERNAME : "extensions.dict.defaultservername",
    DEFAULTSERVERPORT : "extensions.dict.defaultserverport",
    KNOWNSERVERS      : "extensions.dict.knownservers."
};

DictExtension.DictServer = function(name, port, description) {
    this.name        = name;
    this.port        = port;
    this.description = description;
}

// The list of valid DICT servers (generated by mkdictservertable.pl)
DictExtension.DEFAULT_DICT_SERVERS = [
    new DictExtension.DictServer(
        'amos.klf.uw.edu.pl',
        '2628',
        'Amos: a test server for the newest Freedict databases.'),
    new DictExtension.DictServer(
        'dict.antono.info',
        '2628',
        'Esperanto dictionary'),
    new DictExtension.DictServer(
        'dict.arabeyes.org',
        '2628',
        'English-Arabic, extensible'),
    new DictExtension.DictServer(
        'dict.bibleonline.ru',
        '2628',
        'This DICT server Contains The Christian Material.'),
    new DictExtension.DictServer(
        'dict.dvo.ru',
        '2628',
        'Standard databases, russian, ru-en, en-ru dictionaries. Russian web interface.'),
    new DictExtension.DictServer(
        'dict.hewgill.com',
        '2628',
        'Formatted entry pages from the Wiktionary project. Offers four databases with varying levels of detail, including a raw wikitext view.'),
    new DictExtension.DictServer(
        'dict.mova.org',
        '2628',
        'Diverse databases, many Slavic/Russian dictionaries'),
    new DictExtension.DictServer(
        'dict.org',
        '2628',
        'The standard dictd server, contains several databases related to English language and the FreeDict databases'),
    new DictExtension.DictServer(
        'dict.poboxes.info',
        '2628',
        'All standard English databases from dict.org'),
    new DictExtension.DictServer(
        'dict.saugus.net',
        '2628',
        'The contents of the Saugus.net computer knowledgebase, including a section on computer terms, filename extensions, and free software titles.'),
    new DictExtension.DictServer(
        'dict.seesslen.net',
        '2628',
        'Fd-deu-eng, fd-eng-deu, fd-eng-rus'),
    new DictExtension.DictServer(
        'dict.tu-chemnitz.de',
        '2628',
        'Standard databases'),
    new DictExtension.DictServer(
        'dict.tugraz.at',
        '2628',
        'Standard and freedict databases'),
    new DictExtension.DictServer(
        'dict.uni-leipzig.de',
        '2628',
        'Official FreeDict server, most current, OpenSearch description (eg. allows to use this server as a search engine in Firefox)'),
    new DictExtension.DictServer(
        'dictionary.bishopston.net',
        '2628',
        'IPv6 enabled, standard databases (web1913, wn, gazetteer, jargon, foldoc, elements, easton, hitchcock, vera, devils, world95), FreeDict databases'),
    new DictExtension.DictServer(
        'indica-et-buddhica.org',
        '2629',
        'Indica-et-buddhica.org: Sanskrit-Tibetan Buddhist Terminology, Monier-Williams Sanskrit-English Dictionary'),
    new DictExtension.DictServer(
        'la-sorciere.de',
        '2628',
        'Debian terminology dictionaries, standard databases, engdeu'),
    new DictExtension.DictServer(
        'localhost',
        '2628',
        'The DICT server that is running on your local machine.'),
    new DictExtension.DictServer(
        'nb.mblondel.org',
        '2628',
        'A server focusing on free Japanese dictionaries'),
    new DictExtension.DictServer(
        'www.lojban.org',
        '2628',
        'To and from lojban and a number of other languages.'),
];

// Get the default server name, or use a default value
// Does not throw an exception.
DictExtension.getDefaultServerName = function ()
{
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    try {
        return pref.getCharPref(DictExtension.PrefName.DEFAULTSERVERNAME);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return 'dict.org';
}

// Get the default server port, or use a default value
// Does not throw an exception.
DictExtension.getDefaultServerPort = function ()
{
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
        return pref.getCharPref(DictExtension.PrefName.DEFAULTSERVERPORT);
    }
    catch (e) {
    }

    try {
        // maybe it's stored as an int
        return pref.getIntPref(DictExtension.PrefName.DEFAULTSERVERPORT);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return '2628';
}

// Set the server name preference.  Throw an exception if it cannot be set
DictExtension.setDefaultServerName = function(servername)
{
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    pref.setCharPref(DictExtension.PrefName.DEFAULTSERVERNAME, servername);
}

// Set a server port preference.  Throw an exception if it cannot be set
DictExtension.setDefaultServerPort = function(serverport)
{
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
        pref.setCharPref(DictExtension.PrefName.DEFAULTSERVERPORT, serverport);
    }
    catch (e) {
        // maybe it's stored as an int
        pref.setIntPref(DictExtension.PrefName.DEFAULTSERVERPORT, serverport);
    }
}


// Gets the name/port/description of the known server at index i.
// This is a private helper function
DictExtension.getKnownServerInfo = function(index, part)
{
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    var prefName = DictExtension.PrefName.KNOWNSERVERS + index + "." + part;
    return pref.getCharPref(prefName);
}

// Gets the name of the known server at index.
DictExtension.getKnownServerName = function(index)
{
    return DictExtension.getKnownServerInfo(index, "name");
}

// Gets the port of the known server at index.
DictExtension.getKnownServerPort = function(index)
{
    return DictExtension.getKnownServerInfo(index, "port");
}

// Gets the description of the known server at index.
DictExtension.getKnownServerDescription = function(index)
{
    return DictExtension.getKnownServerInfo(index, "description");
}

// Sets the name/port/description of the known server at index to info.
DictExtension.setKnownServerInfo = function(index, part, info)
{
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    var prefName = DictExtension.PrefName.KNOWNSERVERS + index + "." + part;
    pref.setCharPref(prefName, info);
}

// Sets the name of the known server at index.
DictExtension.setKnownServerName = function(index, name)
{
    return DictExtension.setKnownServerInfo(index, "name", name);
}

// Sets the port of the known server at index.
DictExtension.setKnownServerPort = function(index, port)
{
    return DictExtension.setKnownServerInfo(index, "port", port);
}

// Sets the description of the known server at index.
DictExtension.setKnownServerDescription = function(index, description)
{
    return DictExtension.setKnownServerInfo(index, "description", description);
}

// handler for when a new DICT server is selected in the list box
DictExtension.onNewServerSelected = function()
{
    var descriptionElement = document.getElementById("description");

    // remove any existing description
    while (descriptionElement.firstChild) {
        descriptionElement.removeChild(descriptionElement.firstChild);
    }

    var listbox = document.getElementById("dictd-servers");

    var serverID = listbox.selectedIndex;

    // Update the description text to match the description of the
    // newly selected server.  Also update the hidden preference elements.
    var descriptionTextNode;

    try {
        var serverName        = DictExtension.getKnownServerName(serverID);
        var serverPort        = DictExtension.getKnownServerPort(serverID);
        var serverDescription = DictExtension.getKnownServerDescription(serverID);

        // update the preferences in our hidden preference elements
        var defaultServerName = document.getElementById('defaultservername');
        defaultServerName.setAttribute('value', serverName);

        var defaultServerPort = document.getElementById('defaultserverport');
        defaultServerPort.setAttribute('value', serverPort);

        // add the new description
        descriptionTextNode = document.createTextNode(serverDescription);
        descriptionElement.appendChild(descriptionTextNode);
    }
    catch (e) {
        // The selected server is not in the list of known suggested servers.
        // Do not choose one by default and modify the user's
        // preferences, because the user may have hacked their preferences
        // through about:config to point to a server that is not in the
        // known servers list.

        // add the new description
        var descriptionText = 'Please select a server';
        descriptionTextNode = document.createTextNode(descriptionText);
        descriptionElement.appendChild(descriptionTextNode);
    }
}

// handler for when the "Reset to Default" button is pressed
DictExtension.onResetServerChoicesToDefault = function()
{
    // clear the existing settings
    try {
        var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
        pref.deleteBranch(DictExtension.PrefName.KNOWNSERVERS);
    }
    catch (e) {
        // ignore errors
        alert(e);
    }


    // add the default settings
    try {
        var i = 0;
        for (var dictserver in DictExtension.DEFAULT_DICT_SERVERS) {

            DictExtension.setKnownServerName(i, DictExtension.DEFAULT_DICT_SERVERS[dictserver].name);
            DictExtension.setKnownServerPort(i, DictExtension.DEFAULT_DICT_SERVERS[dictserver].port);
            DictExtension.setKnownServerDescription(i, DictExtension.DEFAULT_DICT_SERVERS[dictserver].description);

            i++;
        }
    }
    catch (e) {
        // ignore errors
        alert(e);
    }

    // repopulate the UI elements
    try {
        DictExtension.initPreferences();
    }
    catch (e) {
        // ignore errors
    }
}


DictExtension.initPreferences = function()
{
    // Best-effort to set the preferences to known good values if they cannot be read.
    // This does not throw an exception.
    var serverName;
    var serverPort;

    try {
        serverName = DictExtension.getDefaultServerName();
        DictExtension.setDefaultServerName(serverName);
    }
    catch (e) {
        // ignore errors
    }

    try {
        serverPort = DictExtension.getDefaultServerPort();
        DictExtension.setDefaultServerPort(serverPort);
    }
    catch (e) {
        // ignore errors
    }

    try {
        // try to get info on the 0th server
        serverName = DictExtension.getKnownServerName(0);
        serverPort = DictExtension.getKnownServerPort(0);
    }
    catch (e) {
        // We could not get info on the 0th server.
        // The array must be missing/corrupt.
        // Make a best-effort to restore it
        DictExtension.onResetServerChoicesToDefault();
    }

    var listbox = document.getElementById("dictd-servers");

    // clear the contents of the listbox
    while (listbox.getRowCount() != 0) {
        listbox.removeItemAt(0);
    }

    var i;
    for (i = 0;; i++) {

        try {
            serverName = DictExtension.getKnownServerName(i);
            serverPort = DictExtension.getKnownServerPort(i);

            // append this server to the list box
            var listitem = document.createElement('listitem');
            listitem.setAttribute('value', i);
            listitem.setAttribute('label', serverName);

            listbox.appendChild(listitem);
        }
        catch (e) {
            // we've reached the end of the array
            // (or encountered a malformed entry)
            break;
        }
    }

    // Try to find the index in the known servers that corresponds
    // to the current default server so that we can select it.
    var defaultServerIndex = -1;

    var defaultServerName = DictExtension.getDefaultServerName();
    var defaultServerPort = DictExtension.getDefaultServerPort();

    for (i = 0;; i++) {

        try {
            serverName = DictExtension.getKnownServerName(i);
            serverPort = DictExtension.getKnownServerPort(i);

            if (serverName == defaultServerName &&
                serverPort == defaultServerPort) {
                defaultServerIndex = i;
                break;
            }
        }
        catch (e) {
            // we've reached the end of the array
            // (or encountered a malformed entry)
            break;
        }
    }

    listbox.selectedIndex = defaultServerIndex;
}

DictExtension.savePreferences = function()
{
    try {
        // set the server name
        var defaultServerName = document.getElementById("defaultservername");
        DictExtension.setDefaultServerName(defaultServerName.value);

        // set the server port
        var defaultServerPort = document.getElementById("defaultserverport");
        DictExtension.setDefaultServerPort(defaultServerPort.value);
    }
    catch(e) {
        alert(
            'Unable to save preferences.\n' +
            'If the problem persists, please file a bug report.');
    }
}
